/*
 * Copyright 2021 Esri
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.example.app

import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity

import android.graphics.Color
import android.view.View
import android.widget.ProgressBar
import android.widget.Toast

import java.util.*

import com.esri.arcgisruntime.ArcGISRuntimeEnvironment
import com.esri.arcgisruntime.concurrent.Job
import com.esri.arcgisruntime.geometry.Envelope
import com.esri.arcgisruntime.geometry.SpatialReferences
import com.esri.arcgisruntime.mapping.ArcGISMap
import com.esri.arcgisruntime.mapping.view.Graphic
import com.esri.arcgisruntime.mapping.view.GraphicsOverlay
import com.esri.arcgisruntime.mapping.view.MapView
import com.esri.arcgisruntime.portal.Portal
import com.esri.arcgisruntime.portal.PortalItem
import com.esri.arcgisruntime.symbology.SimpleFillSymbol
import com.esri.arcgisruntime.symbology.SimpleLineSymbol
import com.esri.arcgisruntime.tasks.offlinemap.GenerateOfflineMapParameters
import com.esri.arcgisruntime.tasks.offlinemap.GenerateOfflineMapUpdateMode
import com.esri.arcgisruntime.tasks.offlinemap.OfflineMapTask

import com.example.app.databinding.ActivityMainBinding

class MainActivity : AppCompatActivity() {

    private val activityMainBinding by lazy {
        ActivityMainBinding.inflate(layoutInflater)
    }

    private val mapView: MapView by lazy {
        activityMainBinding.mapView
    }

    private val progressBar: ProgressBar by lazy {
        activityMainBinding.progressBar
    }

    private val areaOverlay: GraphicsOverlay by lazy {
        GraphicsOverlay()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(activityMainBinding.root)

        setApiKeyForApp()

        setupMap()

    }

    override fun onPause() {
        mapView.pause()
        super.onPause()
    }

    override fun onResume() {
        super.onResume()
        mapView.resume()
    }

    override fun onDestroy() {
        mapView.dispose()
        super.onDestroy()
    }

    private fun setApiKeyForApp(){

        ArcGISRuntimeEnvironment.setApiKey("YOUR_ACCESS_TOKEN")

    }

    // set up your map here. You will call this method from onCreate()
    private fun setupMap() {

        // create a portal pointing to ArcGIS Online
        val portal = Portal("https://www.arcgis.com", false)
        // create a portal item for a specific web map id

        val webMapId = "acc027394bc84c2fb04d1ed317aac674"

        val mapItem = PortalItem(portal, webMapId)
        // create the map from the item
        val map = ArcGISMap(mapItem)

        // set the map to the map view
        mapView.map = map

        // add a graphics overlay to the map view
        mapView.graphicsOverlays.add(areaOverlay)

        // create a graphic showing the area of interest
        val offlineArea = createAreaOfInterestGraphic()
        createOfflineMapTaskAndParameters(map, offlineArea)

    }

    private fun createAreaOfInterestGraphic(): Envelope {

        // define area of interest (envelope) to take offline.
        val offlineArea =
            Envelope(-88.1526, 41.7694, -88.1490, 41.7714, SpatialReferences.getWgs84())

        // create a graphic to display the area to take offline
        val lineSymbol = SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, Color.RED, 2f)
        val fillSymbol =
            SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, Color.TRANSPARENT, lineSymbol)
        val offlineAreaGraphic = Graphic(offlineArea, fillSymbol)

        // create a graphics overlay and add the graphic
        areaOverlay.graphics.add(offlineAreaGraphic)

        return offlineArea

    }

    private fun createOfflineMapTaskAndParameters(map: ArcGISMap, offlineArea : Envelope) {

        // create an offline map task using the current map
        val offlineMapTask = OfflineMapTask(map)

        // create a default set of parameters for generating the offline map from the area of interest
        val parametersFuture =
            offlineMapTask.createDefaultGenerateOfflineMapParametersAsync(offlineArea)
        parametersFuture.addDoneListener {
            val parameters = parametersFuture.get().apply {
                updateMode = GenerateOfflineMapUpdateMode.NO_UPDATES
            }
            downloadOfflineMap(offlineMapTask, parameters)
        }

    }

    private fun downloadOfflineMap(offlineMapTask: OfflineMapTask, parameters: GenerateOfflineMapParameters) {

        // Build a folder path named with today's date/time in the "My Documents" folder.
        val downloadLocation =
            getExternalFilesDir(null)?.path + "OfflineMap_" + Calendar.getInstance().time

        val generateJob =
            offlineMapTask.generateOfflineMap(parameters, downloadLocation).apply {

                addProgressChangedListener {
                    progressBar.progress = progress
                }

                addJobDoneListener {
                    // replace the current map with the result offline map when the job finishes
                    if (status == Job.Status.SUCCEEDED) {
                        progressBar.visibility = View.GONE
                        Toast.makeText(
                            this@MainActivity,
                            "Showing offline map.",
                            Toast.LENGTH_LONG
                        ).show()
                        val result = result
                        mapView.map = result.offlineMap
                        areaOverlay.graphics.clear()
                    }
                }

            }

        generateJob.start()
        progressBar.visibility = View.VISIBLE
        Toast.makeText(this@MainActivity, "Starting generate offline map job.", Toast.LENGTH_LONG).show()

    }

}

